/*
 * =====================================================================================
 * 
 *        Filename:  properties.c
 * 
 *     Description:  Vm Applet Properties
 * 
 *         Version:  $Id: properties.c,v 1.4 2006/07/13 13:39:36 kzak Exp $
 *         Created:  21.12.2005 19:58:36 CET
 * 
 *          Author:  Karel Zak <kzak@redhat.com>
 * 
 * =====================================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <config.h>
#include <string.h>

#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <gconf/gconf-client.h>

#include "applet.h"

static void 
vm_prop_delay_cb(GtkWidget *widget, VmAppletCtl *ctl)
{
    _F();
    
    ctl->delay = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)) * 1000;
    vm_timer_refresh(ctl);
    panel_applet_gconf_set_int (PANEL_APPLET(ctl->applet), "delay", ctl->delay, NULL);
}

static void 
vm_prop_mitem_cb(GtkWidget *widget, VmAppletMenuDesc *desc)
{
    VmAppletCtl *ctl = desc->ctl;
	PanelApplet *applet = PANEL_APPLET (ctl->applet);
	gint tmp = GTK_TOGGLE_BUTTON (widget)->active;
    
    _F();
    
	if (tmp != desc->enable)
    {
		desc->enable = (gboolean) tmp;
		panel_applet_gconf_set_bool (applet, desc->gconf, desc->enable, NULL);
        vm_popmenu_destroy_items(ctl);
	}
}

static void
vm_prop_response_cb(GtkDialog *dialog, gint id, VmAppletCtl *ctl)
{
    _F();
    
	if (id == GTK_RESPONSE_HELP) 
    {
		vm_show_help( ctl, NULL );
		return;
	}
	gtk_widget_destroy (GTK_WIDGET (dialog));
	ctl->propwin = NULL;
}

#ifdef ENABLE_CONSOLEHELPER
static void 
vm_prop_termenb_cb(GtkWidget *widget, VmAppletCtl *ctl)
{
	PanelApplet *applet = PANEL_APPLET (ctl->applet);
	gint tmp = GTK_TOGGLE_BUTTON (widget)->active;
    
    _F();
    
	if (tmp != ctl->terminal_enb)
    {
        GtkWidget *entry = (GtkWidget *) g_object_get_data(G_OBJECT(widget), "cmdentry");
        
		ctl->terminal_enb = (gboolean) tmp;
		panel_applet_gconf_set_bool (applet, VM_GCONF_TERMINAL_ENABLE, ctl->terminal_enb, NULL);
        gtk_widget_set_sensitive(entry, ctl->terminal_enb);
	}
}

static gint 
vm_prop_termcmd_cb(GtkWidget *widget, GdkEvent *event, VmAppletCtl *ctl)
{
    PanelApplet *applet = PANEL_APPLET (ctl->applet);
    char *val;
    
    _F();

    val = (char *) gtk_entry_get_text(GTK_ENTRY(widget));
    if (ctl->terminal_cmd)
        g_free(ctl->terminal_cmd);
    ctl->terminal_cmd = val ? g_strdup(val) : NULL;
    panel_applet_gconf_set_string (applet, VM_GCONF_TERMINAL_CMD, ctl->terminal_cmd, NULL);
    return TRUE;
}
#endif /* ENABLE_CONSOLEHELPER */

static gboolean
vm_prop_key_writable (PanelApplet *applet, const char *key)
{
	gboolean writable;
	char *fullkey;
	static GConfClient *client = NULL;
    
    _F();
    
	if (client == NULL)
		client = gconf_client_get_default ();

	fullkey = panel_applet_gconf_get_full_key (applet, key);
	writable = gconf_client_key_is_writable (client, fullkey, NULL);

	g_free (fullkey);
	return writable;
}


static GtkWidget *
vm_prop_spinbutton_new(VmAppletCtl *ctl, GtkWidget *main_box, 
        gchar *title, int val, GCallback callback, char *post, gboolean sen)
{        
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *hbox2;
    GtkWidget *spin;
    
    _F();
    
	hbox = gtk_hbox_new (FALSE, 12);
	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);

    label = gtk_label_new_with_mnemonic (title);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	hbox2 = gtk_hbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (hbox), hbox2, FALSE, FALSE, 0);

    spin = gtk_spin_button_new_with_range(1, 3600, 1);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gdouble) val);
	gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
	gtk_box_pack_start (GTK_BOX (hbox2), spin, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT (spin), "value-changed",  G_CALLBACK (callback), ctl);
	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spin), GTK_UPDATE_ALWAYS);

    if (post)
    {
	    label = gtk_label_new_with_mnemonic (post);
        gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
    }
    
    gtk_widget_set_sensitive (hbox, sen);
    return spin;
}

static GtkWidget *
vm_prop_checkbutton_new(GtkWidget *main_box, char *title, 
        gboolean val, GCallback callback, gpointer cctl, gboolean sen)
{
    GtkWidget *checkbox;
    
    _F();
    
	checkbox = gtk_check_button_new_with_mnemonic (title);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), val);
	g_signal_connect (G_OBJECT (checkbox), "toggled", callback, cctl);
	gtk_box_pack_start (GTK_BOX (main_box), checkbox, FALSE, FALSE, 0);

    gtk_widget_set_sensitive (checkbox, sen);
    return checkbox;
}

static GtkWidget *
vm_prop_entry_new(GtkWidget *main_box, char *title, 
        char *val, GCallback callback, gpointer cctl, gboolean sen)
{
    GtkWidget *entry;
    
    _F();
    
	entry = gtk_entry_new ();
    if (val)
        gtk_entry_set_text (GTK_ENTRY(entry), val);
    
	g_signal_connect (G_OBJECT (entry), "focus-out-event", callback, cctl);
	gtk_box_pack_start (GTK_BOX (main_box), entry, FALSE, FALSE, 0);

    gtk_widget_set_sensitive (entry, sen);
    return entry;
}
   
static GtkWidget *
vm_prop_frame_new(GtkWidget *main_box, gchar *title)
{
	GtkWidget *vbox, *vbox2, *hbox;
	GtkWidget *label;
	gchar *tmp;
		
    _F();
    
	vbox = gtk_vbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (main_box), vbox, FALSE, FALSE, 0);
	
	tmp = g_strdup_printf ("<b>%s</b>", title);
	label = gtk_label_new (NULL);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_label_set_markup (GTK_LABEL (label), tmp);
	g_free (tmp);
	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	
	label = gtk_label_new ("    ");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	
	vbox2 = gtk_vbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);

	return vbox2;	
}

void
vm_prop_show(VmAppletCtl *ctl)
{
    GtkWidget *notebook;
    GtkWidget *vbox;
    GtkWidget *label;
    GtkWidget *frame;
    GtkWidget *foo, *bar;
    gboolean sensitive = TRUE;
    gint i;
        
    _F();

    if (ctl->propwin) 
    {
	    gtk_window_set_screen (GTK_WINDOW (ctl->propwin),
				       gtk_widget_get_screen (ctl->applet));
		gtk_window_present (GTK_WINDOW (ctl->propwin));
		return;
	}

    ctl->propwin = gtk_dialog_new_with_buttons (_("Vm Applet Preferences"), NULL,
						  GTK_DIALOG_DESTROY_WITH_PARENT,
						  GTK_STOCK_HELP, GTK_RESPONSE_HELP,
						  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
						  NULL);
    gtk_window_set_screen (GTK_WINDOW (ctl->propwin),
			       gtk_widget_get_screen (ctl->applet));
    
	g_signal_connect (G_OBJECT (ctl->propwin), "response",
			  G_CALLBACK (vm_prop_response_cb), ctl);
    
	gtk_dialog_set_default_response (GTK_DIALOG (ctl->propwin), GTK_RESPONSE_CLOSE);
  	gtk_dialog_set_has_separator (GTK_DIALOG (ctl->propwin), FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctl->propwin), 5);
	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (ctl->propwin)->vbox), 2);
	gtk_window_set_resizable (GTK_WINDOW (ctl->propwin), FALSE);

	notebook = gtk_notebook_new ();
	gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (ctl->propwin)->vbox), notebook,
			    TRUE, TRUE, 0);
    
	/* PAGE: Display  */
	vbox = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), 12);
    
	label = gtk_label_new (_("Display"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    frame = vm_prop_frame_new (vbox, _("Domain information"));

    /* initialize menu description */
    for (i=0; i < ctl->ndescs; i++)
    {
        VmAppletMenuDesc *desc = &ctl->desc[i];
        
    	sensitive = vm_prop_key_writable (PANEL_APPLET (ctl->applet), desc->gconf);

        vm_prop_checkbutton_new(frame, desc->name, desc->enable, 
                G_CALLBACK(vm_prop_mitem_cb), desc, sensitive);
    }

    /* PAGE: Vm  */
	vbox = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), 12);
    
	label = gtk_label_new (_("Virtual Domains"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    frame = vm_prop_frame_new (vbox, _("Data"));

	/* delay */
	sensitive =  vm_prop_key_writable (PANEL_APPLET (ctl->applet), VM_GCONF_DELAY);
    
    vm_prop_spinbutton_new(ctl, frame, _("U_pdate every:"), ctl->delay / 1000, 
            G_CALLBACK(vm_prop_delay_cb), _("seconds"), sensitive);

#ifdef ENABLE_CONSOLEHELPER
    /* terminal command */
    frame = vm_prop_frame_new (vbox, _("Terminal"));
    
    sensitive = vm_prop_key_writable (PANEL_APPLET (ctl->applet), VM_GCONF_TERMINAL_CMD);

    foo = vm_prop_checkbutton_new(frame, _("Enable terminal, use command:"), ctl->terminal_enb, 
                G_CALLBACK(vm_prop_termenb_cb), ctl, sensitive); 

    bar = vm_prop_entry_new(frame, NULL, ctl->terminal_cmd, 
                G_CALLBACK(vm_prop_termcmd_cb), ctl, 
                sensitive ? (ctl->terminal_enb ? TRUE : FALSE) : FALSE); 
    
    g_object_set_data(G_OBJECT(foo), "cmdentry", (gpointer) bar);
#endif
    
    gtk_widget_show_all(ctl->propwin);
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
